%% scalar conservation lae
%       .
%       u + f(u)' = 0        for u(x,t), 0<x<1, t>0
%                             
%


%% System parameters
% coefficient a as anonymous function
%f = @(u) u.^2/2; 
f = @(u) u.^2./( u.^2 + (1- u).^2); 

%% Discretization of the domain
tend = 0.5;                   %  time interval from t=0 to t=tend

n=100;                      %  n equidistant interior gridpoints
h=1/(n+1);                  %  grid width
x=linspace(0,1,n+2);        %  grid vector including boundary points

% in this version, we specify a maximum Courant number C and calculate time
% step from it
v=diff(f(x))/h; %quick and dirty derivativw
vmax = max(abs(v));
Cmax = 0.5; % set the Courant Number
dt = Cmax*h/vmax ;

%% Initial condition
% initialize with peak around x=0.1
% u = spike(x);

u=zeros(size(x));
u(1) = 1;
plot(x,u);
axis([0 1 -0.1 1]);

%% Time loop
t=0;

% monitor conservation: initial total u
utot=sum(u);

scheme = 'UWC';
while (t< tend-0.5*dt)
    switch scheme
        case 'UWC'
            % explicit Upwind for conservative form
            u(2:n+1) = u(2:n+1) - dt/h*(f(u(2:n+1)) - f(u(1:n)));
            
            
            % Lax-Wendroff conservative forms
            %
            % These schemes are conservative in the interior, but for the
            % spike initial condition some mass flux occurs across the left
            % boundary. The spike should propagate solely to the right,
            % with zero influx across the left boundary, but, to a small
            % amount, the schemes also propagate values to the left,
            % resulting in nonzero flux across the boundary.
        
        case 'RM'
            % Richtmyer two-step LaxWendroff method
            % gives wrong front position for Buckeley-Leverett Problem!
            
            % calculate flux            
            %f = a(x).*u; 
            % half-step
            uhalf = (u(1:n+1)+u(2:n+2))/2 -  dt/(2*h)*(f(u(2:n+2)) - f(u(1:n+1)));
            % new flux for second half-step
            %f = a(x(1:n+1)+h/2).*uhalf;
            % final step
            u(2:n+1) = u(2:n+1) - dt/h*(f(uhalf(2:n+1))-f(uhalf(1:n)));
            

            
    end
    t = t+dt;
    
%     % exact solution
%     uex = spike(x./(exp(t)*(1-x) + x));
%     switch scheme
%         case {'UWC','RM','MC'}
%            uex = uex*exp(t)./(-exp(t)*(x-1)+x).^2;
%     end
    plot(x,u);axis([0 1 -0.1 1]);
    pause(0.01)
    
    
end
